



class Keys(object):
    '''Class to hold various key operations'''


    def autosearch_key(self, args, results):
        '''Search for the correct keyid from the GPGResult'''
        messages = []
        has_no_pubkey, s_keyid = results.no_pubkey
        if has_no_pubkey:
            messages.append(
                _unicode("Auto-searching for key.: 0x%s") % s_keyid)
            # reset all but keyid and pass thru data
            args.keyid = s_keyid
            args.keydir = None
            args.fingerprint = None
            args.exact = False
            args.category = None
            args.nick = None
            args.name = None
            args.all = False
            keys = self.key_search(args, data_only=True)
            if keys:
                args.category = list(keys)[0]
                args.nick = keys[args.category][0].nick
                return (True, args, messages)
            messages.append(_unicode("Failed to find gpg key.: 0x%s") % s_keyid)
        return (False, args, messages)


    def determine_keys(self, args, default_cat=None):
        handler = SeedHandler(self.logger, self.config)
        seeds = handler.load_category(args.category or default_cat)
        keyring = self.config.get_key('keyring')
        catdir = os.path.join(keyring, args.category)
        self.logger.debug(_unicode("LIB: determine_keys; catdir = %s") % catdir)
        kwargs = handler.build_gkeydict(args)
        return seeds.list(**kwargs)


    def key_search(self, args, data_only=False):
        '''Search for a key's seed in the installed keys db'''
        handler = SeedHandler(self.logger, self.config)
        results = {}
        search_args = [x for x in
            ['nick', 'name', 'keydir', 'fingerprint', 'keyid', 'uid']
            if getattr(args, x)]
        if args.category:
            handler.load_category(args.category)
            results[args.category] = handler.key_search(args, search_args)
        else:
            for cat in list(self.config.get_key('seeds')):
                handler.load_category(cat)
                found = handler.key_search(args, search_args)
                if found:
                    if cat in results:
                        results[cat].extend(found)
                    else:
                        results[cat] = found
        keys = {}
        for cat in results:
            keys[cat] = []
            for result in results[cat]:
                if result and result.nick not in keys[cat]:
                    if isinstance(result, GKEY):
                        keys[cat].append(result)
        if data_only:
            del found, cat, handler
            return keys
        msgs = []
        for cat in list(keys):
            msgs.append(_unicode("Category.....: %s") % cat)
            msgs.append(keys[cat])
        del keys, cat
        return (True, msgs)
